//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "ping.h"
#include "emule.h"
#include "preferences.h"
#include "math.h"

IMPLEMENT_DYNCREATE(CPingThread, CWinThread)
CPingThread::CPingThread(void)
{
	isPingThread=false;
	pingStatus.success=false;
	dwAddress=0;
	pHost=NULL;
	hostToPing=0;
	lastCommonTTL=0;
	findHost=false;
	prepares=true;
	searching=true;
	works=false;
	lowestPingIsFound=false;
	autoHost=theApp.glob_prefs->IsAutoHostToPing();
	endPingThread = new CEvent();
	m_bAutoDelete=false;
	lastRestartTime=0;
	failedTimes=0;
	timeOut = TIMEOUT;
	lastCommonRouteFinder = NULL;
	pinger = NULL;
}

CPingThread::~CPingThread(void)
{
     if (pinger)
	delete pinger;
	// ZZ:SpeedSense -->
	if (lastCommonRouteFinder)
    delete lastCommonRouteFinder;
	// ZZ:SpeedSense <--
	if (endPingThread)
	delete endPingThread;
}


void CPingThread::Init(void)
{
	avg_ping=0.1f;
	lowest_ping=0.1f;
	lowest_index=0;
	for (int i=0; i<m_lowest_ping_values; i++)
		lowest_list[i]=0.0f;
	
	if (theApp.glob_prefs->IsUDPing())
		Pinger::sharedPingMode = pingAuto;
	else
		Pinger::sharedPingMode = pingICMP;

        if (pinger)
            delete pinger;
	pinger = new Pinger();
	// ZZ:SpeedSense -->
    lastCommonRouteFinder = new LastCommonRouteFinder();
	// ZZ:SpeedSense <--
}

pingVal CPingThread::GetPing(){
	pingVal ret={avg_ping, changed, lowest_ping, lowest_changed}; 
	changed=false;
	lowest_changed=false;
	return ret;
}

void CPingThread::findHostToPing(){
	if (lastCommonRouteFinder->Find()){
		hostToPing = lastCommonRouteFinder->GetHostToPing();
		lastCommonTTL = lastCommonRouteFinder->GetLastCommonTTL();
		lastCommonHost = lastCommonRouteFinder->GetLastCommonHost();
	}
	else {
		hostToPing = 0;
		lastCommonTTL = 0;
		lastCommonHost = 0;
	}
}

// ZZ:SpeedSense -->
void CPingThread::findLowestPing(){
	theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Finding a start value for lowest ping...");

	uint32 toPing=0;
	uint32 TTL=0;
	uint32 fails=0;
	
	float tempVals;

	if (hostToPing && autoHost){
		toPing=hostToPing;
		TTL=lastCommonTTL;
	}
	else if (dwAddress){
		toPing=dwAddress;
		TTL=DEFAULT_TTL;
	}
	else {
		works = false;
		theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Failed calculeting lowest ping - no host to ping, check configuration. Restarting SpeedSense...");
		return;
	}
	// finding lowest ping
	tempVals=(float)TIMEOUT;
	for(int initialPingCounter = 0; isPingThread && initialPingCounter < 10; initialPingCounter++) {
		Sleep(200);

		pingStatus = pinger->Ping(toPing, TTL, true);

		if (pingStatus.success && (pinger->PingMode() == pingICMP || pinger->PingMode() == pingUDP && !pinger->IsTimeOut()))
			tempVals=min(tempVals, pingStatus.delay);
		else {
			fails++;
			theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Ping #%i failed. Reason follows", initialPingCounter);
			pinger->PIcmpErr(pingStatus.error);
		}
	}

	if (fails>=3){
		works=false;
		theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Failed calculeting lowest ping - too many fails. Restarting SpeedSense...");
		return;
	}

	if (tempVals < 0.1f)
		tempVals = 0.1f;

	lowest_ping = tempVals;
	for (int i=0; i<m_lowest_ping_values; i++)
		lowest_list[i]=lowest_ping;
	timeOut = 800 * log10f(8+lowest_ping*lowest_ping);
	theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Lowest ping: %3.2f ms", lowest_ping);
}
// ZZ:SpeedSense <--

int CPingThread::Run(void)
{
	float temp_lowest_ping;
	uint32 start;
	uint32 end;

	endPingThread->ResetEvent();

	while (isPingThread) {
		start=::GetTickCount();

		// Check if user has changed the option in preference (auto/manual host)
		if (autoHost != theApp.glob_prefs->IsAutoHostToPing()){
			autoHost = theApp.glob_prefs->IsAutoHostToPing();
			if (autoHost)
				findHost=true;
			lowestPingIsFound=false;
		}
		// Check if pinging failed and there is a need to restart
		if (!works && ::GetTickCount()-lastRestartTime > 10000){
			searching=true;
			theApp.emuledlg->UpdateBCStatus();
			if (autoHost) findHostToPing();
			if (Pinger::sharedPingMode == pingICMP) 
				SetIP(theApp.glob_prefs->GetURLPing());
			else
				dwAddress = 0;
			if (dwAddress || (hostToPing  && autoHost)){
				works=true;
				theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Initialized succesfully.");
			}
			else {
				Pinger::sharedPingMode = ((Pinger::sharedPingMode == pingICMP) ? pingAuto : pingICMP);
				if (Pinger::sharedPingMode == pingAuto)
					dwAddress = 0;
				delete pinger;
				pinger = new Pinger();
			}
			searching=false;
			if (lowestPingIsFound || !works){
				prepares=false;
				theApp.emuledlg->UpdateBCStatus();
			}
			findHost=false;
			lastRestartTime=::GetTickCount();
		}
		// Check if network topology changed then find new host
		else if (findHost) {
			searching=true;
			theApp.emuledlg->UpdateBCStatus();
			if (autoHost) findHostToPing();
			if (hostToPing){
				works=true;
				theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Initialized succesfully.");
			}
			else {
				Pinger::sharedPingMode = ((Pinger::sharedPingMode == pingICMP) ? pingAuto : pingICMP);
				if (Pinger::sharedPingMode == pingAuto)
					dwAddress = 0;
				delete pinger;
				pinger = new Pinger();
			}
			searching=false;
			if (lowestPingIsFound || !works){
				prepares=false;
				theApp.emuledlg->UpdateBCStatus();
			}
			findHost=false;
		}
		
		// Check the need to find out of lowest ping
		if (!lowestPingIsFound && works){
			if (!prepares){
				prepares=true;
				theApp.emuledlg->UpdateBCStatus();
				::Sleep(1000);
			}
			findLowestPing();
			if (works){
				temp_lowest_ping=lowest_ping;
				lowestPingIsFound=true;
				lowest_changed=true;
			}
			prepares=false;
			theApp.emuledlg->UpdateBCStatus();
		}

		// Send new ICMP packet
		if (works) {
			avg_ping = Ping();
			changed = true;

			// Check lowest ping
			if (avg_ping<lowest_ping && (pinger->PingMode() == pingICMP || pinger->PingMode() == pingUDP && !pinger->IsTimeOut())){
				temp_lowest_ping = lowest_ping * m_lowest_ping_values - lowest_list[lowest_index];
				lowest_list[lowest_index]=avg_ping;
				temp_lowest_ping += lowest_list[lowest_index];
				lowest_ping = temp_lowest_ping/m_lowest_ping_values;
				lowest_changed=true;
				lowest_index++;
				lowest_index%=m_lowest_ping_values;
				timeOut = 800 * log10f(8+lowest_ping*lowest_ping);
				theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Lowest ping: %3.2f ms", lowest_ping);
			}
			// End check
			
			// ZZ:SpeedSense -->
		}
        uint32 ticksBetweenPings = 1000;
		float upload=theApp.BC->GetMaxUpload();
        if(upload > 0.0f) {
			// UDP pinger 20 /IP/ + 8 /UDP/ + 4 /Data/ = 32 bytes
			if (pinger->PingMode() == pingUDP)
				ticksBetweenPings = 3200/upload; //4*100/upload
            // ping packages being 64 bytes, this should use 1% of bandwidth (one hundredth of bw).
			else
				ticksBetweenPings = 6400/upload; //64*100/upload

            if(ticksBetweenPings < 125) {
                // never ping more than 8 packages a second
                ticksBetweenPings = 125;
            } else if(ticksBetweenPings > 1000) {
                ticksBetweenPings = 1000;
            }
        }
		// ZZ:SpeedSense <--
 		end=::GetTickCount();

		// Sleep before next loop
		if ((end-=start) < ticksBetweenPings)
			::Sleep(ticksBetweenPings-end);
	}

	endPingThread->SetEvent();
	return 0;
}

CString	 CPingThread::GetPingedServer(){
	if (isPreparing() || !lowestPingIsFound || autoHost != theApp.glob_prefs->IsAutoHostToPing())
		return "-";
	if (theApp.glob_prefs->IsAutoHostToPing() && lastCommonHost){
			IN_ADDR stLastCommonHostAddr;
			stLastCommonHostAddr.s_addr = lastCommonHost;

			CString ret;
			ret.Format("%s",inet_ntoa(stLastCommonHostAddr));
			return ret;
	}
	if (dwAddress)
		return theApp.glob_prefs->GetURLPing();
	return "Failed - bad address";
}

bool CPingThread::SetIP(CString pingThreadNic)
{	
	dwAddress = 0;
	// Lookup destination
    // Use inet_addr() to determine if we're dealing with a name
    // or an address
	if (pingThreadNic=="")
		return false;
	iaDest.s_addr = inet_addr(pingThreadNic);
	if (iaDest.s_addr == INADDR_NONE) {
		pHost = gethostbyname(pingThreadNic);
	} else {
        pHost = gethostbyaddr((const char*)&iaDest, sizeof(struct in_addr), AF_INET);
	}
	if (pHost == NULL) {
		theApp.emuledlg->QueueDebugLogLine(false, "\n%s not found\n", pingThreadNic);
		return false;
	}
	dwAddress = *(DWORD*)(*pHost->h_addr_list);

	if (!autoHost || !hostToPing)
		lowestPingIsFound=false;
	return true;
}

float CPingThread::Ping(void)
{
	if (hostToPing && autoHost) {
		pingStatus = pinger->Ping(hostToPing,lastCommonTTL,false, timeOut);
		if(pingStatus.success && pingStatus.destinationAddress != lastCommonHost) {
			// something has changed about the topology! We got another ip back from this ttl than expected.
			// Do the tracerouting again to figure out new topology
			IN_ADDR stLastCommonHostAddr;
			stLastCommonHostAddr.s_addr = lastCommonHost;

			IN_ADDR stDestinationAddr;
			stDestinationAddr.s_addr = pingStatus.destinationAddress;

			theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: Network topology has changed. TTL: %i Expected ip: %s Got ip: %s Will do a new traceroute.", lastCommonTTL, inet_ntoa(stLastCommonHostAddr), inet_ntoa(stDestinationAddr));
			findHost = true;
			lowestPingIsFound = false;
		}
	}
	else if (dwAddress)
		pingStatus = pinger->Ping(dwAddress,DEFAULT_TTL,false, timeOut);
	else {
		works=false;
		theApp.emuledlg->QueueDebugLogLine(false,"No host to ping, check configuration. Restarting SpeedSense.");
		theApp.emuledlg->UpdateBCStatus();
	}

	if (pingStatus.success || pingStatus.error == IP_REQ_TIMED_OUT){
		failedTimes = 0;

		if (pingStatus.delay < 0.1f)
			pingStatus.delay = 0.1f;

		return pingStatus.delay;
	}
	else {
		failedTimes++;
		if (failedTimes == 3){
			failedTimes = 0;
			works=false;
			lowestPingIsFound=false;
			theApp.emuledlg->QueueDebugLogLine(false,"SpeedSense: failed too many times. Restarting SpeedSense...");
			theApp.emuledlg->UpdateBCStatus();
		}
		return timeOut;
	}
	return timeOut;
}

// Stop thread (can be called more than one time)
void CPingThread::StopPingThread(void)
{
	if (isPingThread) {
		SuspendThread();
		isPingThread = false; 
	}
}

// Resumes thread (can be called more than one time)
void CPingThread::StartPingThread(void)
{
	if (!isPingThread) {
		isPingThread = true;
		ResumeThread();
	}
}

// Finish thread
void CPingThread::EndPingThread(void)
{
	isPingThread = false;
	lastCommonRouteFinder->StopSearching();
	ResumeThread();
	endPingThread->Lock();
}
